class CreateDomainDefinition:
    name: str
    userspace: str
    parent_id: int
    reference: str

    def __init__(self,
                 name: str,
                 userspace: str,
                 parent_id: int,
                 reference: str = ''):
        self.name = name
        self.userspace = userspace
        self.parent_id = parent_id
        self.reference = reference

    def __iter__(self):
        yield "name", self.name
        yield "userspace", self.userspace
        yield "parentid", self.parent_id
        yield "reference", self.reference


class UpdateCourseDefinition:
    course_id: str
    domain_id: str
    title: str
    reference: str

    def __init__(self,
                 course_id: str,
                 domain_id: str,
                 title: str,
                 reference: str):
        self.course_id = course_id
        self.domain_id = domain_id
        self.title = title
        self.reference = reference


    def __iter__(self):
        yield "courseid", self.course_id
        yield "domainid", self.domain_id
        yield "title", self.title
        yield "reference", self.reference


class CopyCourseDefinition:
    course_id: str
    domain_id: str
    action: str
    reference: str
    status: int

    def __init__(self,
                 course_id: str,
                 domain_id: str,
                 action: str = 'StaticCopy',
                 reference: str = '',
                 status: int = 0):
        self.course_id = course_id
        self.domain_id = domain_id
        self.action = action
        self.reference = reference
        self.status = status


from requests import Session
from urllib.parse import urlencode

class Connection:
    token: str
    session: Session
    base_url: str
    home_domain_id: int

    def __init__(self,
                 base_url: str,
                 home_domain_id: int,
                 token: str = "",
                 session: Session = Session()):
        self.token = token
        self.base_url = base_url
        self.session = session
        self.home_domain_id = home_domain_id


    def get(self,
            cmd: str,
            params: dict = {}):
        query_string = urlencode({ 'cmd': cmd, '_token': self.token } | params, doseq=True)

        response = self.session.get(f"{self.base_url}?{query_string}")
        response.raise_for_status()

        if response.status_code != 200:
            raise Exception(f"Error: {response.status_code} - {response.text}")

        return response.json()

    def post(self,
             cmd: str,
             params: dict = {},
             payload: dict = {}):
        query_string = ""

        if cmd:
            query_string = urlencode({ 'cmd': cmd, '_token': self.token } | params, doseq=True)
        else:
            payload = {'_token': self.token} | payload
            if bool(params):
                query_string = urlencode(params, doseq=True)

        target_url = f"{self.base_url}?{query_string}" if query_string else self.base_url

        # NEED TO DO A POST PAYLOAD WITH THE TOKEN ADDED TO THE REQUESTS OBJECT

        response = self.session.post(target_url, json=payload)
        response.raise_for_status()

        return response.json()


from agilix_api_fr8train.models.domains import CreateDomainDefinition


def create_domain_list_to_payload(domain_list: list[CreateDomainDefinition]) -> dict:
    return {"requests": {"domain": list(map(lambda x: dict(x), domain_list))}}
